<?php

namespace Drupal\forgot_username\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form for /user/username.
 */
class ForgotUsernameForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['email_address'] = [
      '#type' => 'textfield',
      '#title' => t('Your e-mail address'),
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Request username'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user.username';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableNames() {
    return 'forgot_username';
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $email = $form_state->getValue('email_address');
    $database = \Drupal::database();
    $query = $database->select('users_field_data', 'u');
    $query->fields('u', ['name']);
    $query->condition('u.mail', $email, '=');
    $user = $query->execute()->fetchAll();
    if (!\Drupal::service('email.validator')->isValid($email)) {
      $form_state->setErrorByName("email_address", t("The e-mail address is not valid."));
    }
    if (!$user) {
      $form_state->setErrorByName("email_address", t("There is no account with that e-mail address."));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $users = \Drupal::entityTypeManager()->getStorage('user')->loadByProperties(['mail' => $form_state->getValue('email_address')]);
    $account = reset($users);
    $mail = _user_mail_notify('username_reset', $account);
    if (!empty($mail)) {
      $message = $this->t('Your username has been e-mailed to the address specified. The e-mail might end up in spam folder, so please check there before complaining.');
      \Drupal::messenger()->addMessage($message);
    }
  }

}
